use anyhow::Context;
use forgejo_api::structs::{CreateIssueCommentOption, PullRequest, RepoListPullRequestsQuery};

use crate::cli::pull_request::comment::CommentPullRequestArgs;

use crate::git_info::repo_owner::RepoAndOwner;
use crate::render::option::option_display;
use crate::render::spinner::spin_until_ready;
use crate::render::ui::fuzzy_select_with_key;

use crate::actions::text_manipulation::{edit_prompt_for, select_prompt_for};
use crate::types::context::BergContext;

use super::display_pull_request;

pub async fn comment_pull(ctx: BergContext<CommentPullRequestArgs>) -> anyhow::Result<()> {
    let RepoAndOwner { repo, owner } = ctx.repo_owner()?;
    let pull_request = select_pull_request(&ctx).await?;
    let pull_request_id = pull_request
        .id
        .context("Selected milestone doesn't have an ID")?;

    let options = create_options(&ctx, &pull_request).await?;

    let comment = ctx
        .client
        .issue_create_comment(owner.as_str(), repo.as_str(), pull_request_id, options)
        .await?;

    tracing::debug!("{comment:?}");

    Ok(())
}

async fn create_options(
    _ctx: &BergContext<CommentPullRequestArgs>,
    pull_request: &PullRequest,
) -> anyhow::Result<CreateIssueCommentOption> {
    let body = inquire::Editor::new(edit_prompt_for("a comment").as_str())
        .with_predefined_text(
            format!(
                "Write a comment for pull_request \"{}\"",
                option_display(&pull_request.title)
            )
            .as_str(),
        )
        .prompt()?;

    Ok(CreateIssueCommentOption {
        body,
        updated_at: None,
    })
}

async fn select_pull_request(
    ctx: &BergContext<CommentPullRequestArgs>,
) -> anyhow::Result<PullRequest> {
    let RepoAndOwner { repo, owner } = ctx.repo_owner()?;
    let pull_requests_list = spin_until_ready(ctx.client.repo_list_pull_requests(
        owner.as_str(),
        repo.as_str(),
        RepoListPullRequestsQuery::default(),
    ))
    .await?;

    fuzzy_select_with_key(
        &pull_requests_list,
        select_prompt_for("pull request"),
        display_pull_request,
    )
    .cloned()
}
