use anyhow::Context;
use url::Url;

use crate::actions::repo::parse_owner_and_repo;
use crate::cli::repo::clone::RepoCloneArgs;
use crate::render::spinner::spin_until_ready;
use crate::render::ui::confirm_with_prompt;
use crate::types::context::BergContext;

pub async fn clone_repo(ctx: BergContext<RepoCloneArgs>) -> anyhow::Result<()> {
    let (owner, repo) = parse_owner_and_repo(ctx.args.owner_and_repo.as_str())?;
    let url = if ctx.args.use_ssh {
        anyhow::bail!("SSH Cloning under maintenance");
        #[allow(unreachable_code)]
        spin_until_ready(get_ssh_url(&ctx, owner.as_str(), repo.as_str())).await?
    } else {
        ctx.config
            .url()?
            .join(format!("/{owner}/{repo}.git").as_str())?
    };
    ask_confirm_clone(repo.as_str())?;
    start_clone_repo(url)?;
    Ok(())
}

async fn get_ssh_url(
    ctx: &BergContext<RepoCloneArgs>,
    ownername: &str,
    reponame: &str,
) -> anyhow::Result<Url> {
    ctx.client
        .repo_get(ownername, reponame)
        .await
        .map_err(anyhow::Error::from)
        .inspect(|x| tracing::debug!("{x:#?}"))
        .and_then(|repo| repo.ssh_url.context("No SSH url on repo"))
        .and_then(|url| Url::parse(url.as_str()).context("Url wasn't a valid SSH Url"))
        .context("User doesn't own the repo that was specified.")
}

fn ask_confirm_clone(reponame: &str) -> anyhow::Result<()> {
    let current_path = std::env::current_dir()?;
    if !confirm_with_prompt(
        format!("Do you really to clone {reponame} into the directory {current_path:?}").as_str(),
    )? {
        anyhow::bail!("Abort cloning the repository.")
    }
    Ok(())
}

fn start_clone_repo(ssh_url: Url) -> anyhow::Result<()> {
    let mut cmd = std::process::Command::new("git");
    cmd.arg("clone").arg(ssh_url.as_str());
    tracing::debug!("cmd: {cmd:?}");
    let mut child = cmd.stdout(std::process::Stdio::inherit()).spawn()?;
    child.wait()?;
    Ok(())
}
