use crate::actions::auth::login::login_user;
use crate::actions::auth::logout::logout_user;
use crate::actions::config::generate::generate_config;
use crate::actions::config::info::info_config;
use crate::actions::issue::comment::comment_issue;
use crate::actions::issue::create::create_issue;
use crate::actions::issue::edit::edit_issue;
use crate::actions::issue::list::list_issue;
use crate::actions::issue::view::view_issue;
use crate::actions::label::create::create_label;
use crate::actions::label::delete::delete_label;
use crate::actions::label::edit::edit_label;
use crate::actions::label::list::list_label;
use crate::actions::milestone::create::create_milestone;
use crate::actions::milestone::edit::edit_milestone;
use crate::actions::milestone::list::list_milestone;
use crate::actions::milestone::view::view_milestone;
use crate::actions::notification::list::list_notifications;
use crate::actions::notification::view::view_notifications;
use crate::actions::pull_request::comment::comment_pull;
use crate::actions::pull_request::create::create_pull;
use crate::actions::pull_request::edit::edit_pull;
use crate::actions::pull_request::list::list_pull;
use crate::actions::pull_request::view::view_pull;
use crate::actions::repo::assignees::assignees_repo;
use crate::actions::repo::clone::clone_repo;
use crate::actions::repo::create::create_repo;
use crate::actions::repo::delete::delete_repo;
use crate::actions::repo::fork::fork_repo;
use crate::actions::repo::info::info_repo;
use crate::actions::user::info::info_user;
use crate::cli::api::ApiArgs;
use crate::cli::auth::AuthArgs;
use crate::cli::config::ConfigArgs;
use crate::cli::issue::IssueArgs;
use crate::cli::label::LabelArgs;
use crate::cli::milestone::MilestoneArgs;
use crate::cli::notification::NotificationArgs;
use crate::cli::pull_request::PullRequestArgs;
use crate::cli::repo::RepoArgs;
use crate::cli::user::UserArgs;
use crate::cli::BergActions;
use crate::types::config::BergConfig;
use crate::{actions::api::version::print_api_version, cli::BergCommand};

use crate::types::context::BergContext;

use clap::Parser;

pub async fn berg_main() {
    tracing_subscriber::fmt::init();

    if let Err(e) = run(BergCommand::parse()).await {
        let config = match BergConfig::new() {
            Ok(config) => config,
            Err(e) => {
                eprintln!("{e:?}");
                return;
            }
        };
        let mut table = config.make_table();
        table.add_row(vec![
            comfy_table::Cell::new(format!("{e:?}")).fg(comfy_table::Color::Red)
        ]);
        eprintln!("{table}");
    }
}

async fn run(cli: BergCommand) -> anyhow::Result<()> {
    tracing::debug!("args: {cli:#?}");
    use BergActions::*;
    match cli.sub {
        Auth(AuthArgs::Login(login_args)) => login_user(login_args).await,
        Completion { shell } => BergCommand::generate_completion(shell),
        Config(ConfigArgs::Info(args)) => info_config(args),
        Config(ConfigArgs::Generate(args)) => generate_config(args),
        API(ApiArgs::Version(args)) => print_api_version(BergContext::new(args).await?).await,
        Auth(AuthArgs::Logout(args)) => logout_user(BergContext::new(args).await?),
        User(UserArgs::Info(args)) => info_user(BergContext::new(args).await?).await,
        Repo(RepoArgs::Info(args)) => info_repo(BergContext::new(args).await?).await,
        Repo(RepoArgs::Assignees(args)) => assignees_repo(BergContext::new(args).await?).await,
        Repo(RepoArgs::Create(args)) => create_repo(BergContext::new(args).await?).await,
        Repo(RepoArgs::Clone(args)) => clone_repo(BergContext::new(args).await?).await,
        Repo(RepoArgs::Delete(args)) => delete_repo(BergContext::new(args).await?).await,
        Repo(RepoArgs::Fork(args)) => fork_repo(BergContext::new(args).await?).await,
        Issue(IssueArgs::List(args)) => list_issue(BergContext::new(args).await?).await,
        Issue(IssueArgs::Create(args)) => create_issue(BergContext::new(args).await?).await,
        Issue(IssueArgs::View(args)) => view_issue(BergContext::new(args).await?).await,
        Issue(IssueArgs::Edit(args)) => edit_issue(BergContext::new(args).await?).await,
        Issue(IssueArgs::Comment(args)) => comment_issue(BergContext::new(args).await?).await,
        Pull(PullRequestArgs::List(args)) => list_pull(BergContext::new(args).await?).await,
        Pull(PullRequestArgs::Create(args)) => create_pull(BergContext::new(args).await?).await,
        Pull(PullRequestArgs::Edit(args)) => edit_pull(BergContext::new(args).await?).await,
        Pull(PullRequestArgs::View(args)) => view_pull(BergContext::new(args).await?).await,
        Pull(PullRequestArgs::Comment(args)) => comment_pull(BergContext::new(args).await?).await,
        Label(LabelArgs::List(args)) => list_label(BergContext::new(args).await?).await,
        Label(LabelArgs::Create(args)) => create_label(BergContext::new(args).await?).await,
        Label(LabelArgs::Delete(args)) => delete_label(BergContext::new(args).await?).await,
        Label(LabelArgs::Edit(args)) => edit_label(BergContext::new(args).await?).await,
        Milestone(MilestoneArgs::List(args)) => list_milestone(BergContext::new(args).await?).await,
        Milestone(MilestoneArgs::View(args)) => view_milestone(BergContext::new(args).await?).await,
        Milestone(MilestoneArgs::Create(args)) => {
            create_milestone(BergContext::new(args).await?).await
        }
        Milestone(MilestoneArgs::Edit(args)) => edit_milestone(BergContext::new(args).await?).await,
        Notification(NotificationArgs::List(args)) => {
            list_notifications(BergContext::new(args).await?).await
        }
        Notification(NotificationArgs::View(args)) => {
            view_notifications(BergContext::new(args).await?).await
        }
    }
}
