use anyhow::Context;
use comfy_table::Table;

use crate::client::BergClient;
use crate::git_info::repo_owner::{get_repo_owner, RepoAndOwner};
use crate::types::config::BergConfig;
use crate::types::token::Token;

pub struct BergContext<T> {
    pub client: BergClient,
    pub config: BergConfig,
    pub repo_owner: Option<RepoAndOwner>,
    pub args: T,
}

impl<T> BergContext<T> {
    pub async fn new(args: T) -> anyhow::Result<Self> {
        let config = BergConfig::new()?;
        let token = Token::read_from_data_dir().context(
            "Couldn't find login data. Please use `berg auth login` to authenticate first.",
        )?;
        tracing::debug!("{token:?}");
        let base_url = config.url()?;
        tracing::debug!("{base_url:#?}");
        let client = BergClient::new(&token, base_url)?;

        let version = client.get_version().await?;
        tracing::debug!("API VERSION: {version:?}");

        Ok(Self {
            client,
            config,
            repo_owner: get_repo_owner().ok(),
            args,
        })
    }

    pub fn repo_owner(&self) -> anyhow::Result<RepoAndOwner> {
        self.repo_owner
            .as_ref()
            .cloned()
            .map(|repo_owner| {
                tracing::debug!("{repo_owner:?}");
                repo_owner
            })
            .context("Repo and Owner Needed for this operation")
    }

    pub fn make_table(&self) -> Table {
        self.config.make_table()
    }
}
